#include <gint/display.h>
#include <gint/keyboard.h>
#include <gint/gint.h>
#include <gint/bfile.h>
#include <dirent.h>

#ifdef FX9860G
#define LINE_HEIGHT 8
#endif

#ifdef FXCG50
#define LINE_HEIGHT 14
#endif

static void print(int line, char const *str)
{
    dtext(1, line * LINE_HEIGHT, C_BLACK, str);
}

static void listdir_dirent(char const *path)
{
    DIR *dp = opendir(path);
    if(!dp)
        return print(0, "error");

    int line = 0;
    struct dirent *ent;
    while((ent = readdir(dp))) {
        print(line, ent->d_name);
        line++;
    }
}

static void fc_to_utf8(char *utf8, uint16_t const *fc, size_t utf8_len)
{
    size_t j = 0;

    for(size_t i = 0; j < utf8_len && fc[i] != 0 && fc[i] != 0xffff; i++) {
        if(fc[i] == '\\')
            utf8[j++] = '/';
        else if(fc[i] <= 0x7f)
            utf8[j++] = fc[i];
    }

    if(j < utf8_len)
        utf8[j++] = 0;
}

static void listdir_bfile(uint16_t const *search)
{
    struct BFile_FileInfo info;
    uint16_t fc_path[512];
    char utf8_path[512];

    int sd = -1, line = 0;
    int rc = BFile_FindFirst(search, &sd, fc_path, &info);
    if(rc < 0)
        return print(0, "error");

    do {
        fc_to_utf8(utf8_path, fc_path, sizeof utf8_path);
        print(line, utf8_path);
        line++;

        rc = BFile_FindNext(sd, fc_path, &info);
    }
    while(rc >= 0);

    BFile_FindClose(sd);
}

int main(void)
{
    dclear(C_WHITE);

    // gint_world_switch(GINT_CALL(listdir_dirent, "/"));
    gint_world_switch(GINT_CALL(listdir_bfile, u"\\\\fls0\\*"));

    dupdate();
    getkey();
    return 1;
}
